/*-*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-



  ***** BEGIN LICENSE BLOCK *****

* Version: MPL 1.1/GPL 2.0/LGPL 2.1

*

* The contents of this file are subject to the Mozilla Public License Version

* 1.1 (the "License"); you may not use this file except in compliance with

* the License. You may obtain a copy of the License at

* http://www.mozilla.org/MPL/

*

* Software distributed under the License is distributed on an "AS IS" basis,

* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License

* for the specific language governing rights and limitations under the

* License.

*

* The Original Code is:

* ChattyTunes - a ChatZilla Media Player plugin utilizing the FoxyTunes

* Firefox Extension by Alex Sirota (alex@elbrus.com)

*

* The Initial Developers of the Original Code are:

* David Corry <dmcorry@gmail.com> and Gijs Kruitbosch <gk1987@gmail.com>.

*

* Portions created by David Corry and Gijs Kruitbosch are Copyright (C) 2005

* David Corry and Gijs Kruitbosch. All Rights Reserved.

*

* Contributor(s): David Corry & Gijs Kruitbosch

*

* Alternatively, the contents of this file may be used under the terms of

* either the GNU General Public License Version 2 or later (the "GPL"), or

* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),

* in which case the provisions of the GPL or the LGPL are applicable instead

* of those above. If you wish to allow use of your version of this file only

* under the terms of either the GPL or the LGPL, and not to allow others to

* use your version of this file under the terms of the MPL, indicate your

* decision by deleting the provisions above and replace them with the notice

* and other provisions required by the LGPL or the GPL. If you do not delete

* the provisions above, a recipient may use your version of this file under

* the terms of any one of the MPL, the GPL or the LGPL.

*

*

* FoxyTunes Extension and Engine - Copyright (C) 2005 Alex Sirota <alex@elbrus.com>.

* All Rights Reserved.

* FoxyTunes Home Page - http://www.foxytunes.org/

*

* FoxyTunes extension is used with written permission from its author.

*

* A big "thank you" to the chatzilla irc channel members, for helping us

* whenever we needed it :-).

*

****** END LICENSE BLOCK ***** */



plugin.id = "ChattyTunes";

const CT_ERROR = "tunes-error";

const CT_PLAY = "tunes-play";

const CT_INFO = "tunes-info";

const CT_PAUSE = "tunes-pause";

const CT_STOP = "tunes-stop";



plugin.init =

function initPlugin (glob)

{

    //Setting up any global things that might come of use to anybody.

    plugin.major = 0;

    plugin.minor = 7;

    plugin.version = plugin.major + "." + plugin.minor;

    plugin.urlString = String(plugin.url).replace(/\/[^\/]*$/, "/");

    plugin.bundleURL = plugin.urlString + "locale/locale.properties";

    plugin.bundle = client.messageManager.addBundle(plugin.bundleURL);

    plugin.description = MSG_CHATTYTUNES_DESCRIPTION;



    //Player map.

    plugin.playersMap = {

        iTunes: {

            shortName: "iTunes",

            accessKey: "u",

            longName: "iTunes"

        },

        WindowsMediaPlayer: {

            shortName: "WMP",

            accessKey: "W",

            longName: "Windows Media Player"

        },

        WinAmp: {

            shortName: "Winamp",

            accessKey: "W",

            longName: "Winamp"

        },

        WinAmp3: {

            shortName: "Winamp",

            accessKey: "W",

            longName: "Winamp 3"

        },

        XMPlay: {

            shortName: "XMPlay",

            accessKey: "X",

            longName: "XMPlay"

        },

        JetAudio: {

            shortName: "jetAudio",

            accessKey: "A",

            longName: "jetAudio"

        },

        Apollo: {

            shortName: "Apollo",

            accessKey: "a",

            longName: "Apollo",

        },

        MusicMatchJukeBox: {

            shortName: "MusicMatch",

            accessKey: "M",

            longName: "MusicMatch Jukebox"

        },

        wxMusik: {

            shortName: "wxMusik",

            accessKey: "w",

            longName: "wxMusik"

        },

        GenericPlayer: {

            shortName: "Tunes",

            accessKey: "u",

            longName: "my music player"

        },

        foobar2000: {

            shortName: "foobar2000",

            accessKey: "b",

            longName: "foobar2000"

        },

        MediaMonkey: {

            shortName: "MediaMonkey",

            accessKey: "M",

            longName: "MediaMonkey"

        },

        JRiverMediaCenter: {

            shortName: "JRiver",

            accessKey: "J",

            longName: "JRiver Media Center",

        },

        Sonique: {

            shortName: "Sonique",

            accessKey: "S",

            longName: "Sonique"

        },

        TheCoreMediaPlayer: {

            shortName: "CoreMedia",

            accessKey: "C",

            longName: "The Core Media Player"

        },

        QuintessentialPlayer: {

            shortName: "QCD",

            accessKey: "Q",

            longName: "Quintessential Player"

        }

    }



    //Backup plugin ID

    plugin.pluginID = "GenericPlayer";



    //Prefrences ahoy!

    var myprefs = [

        ["NPFormat", MSG_CHATTYTUNES_DEFAULTNPFORMAT, ""],

        ["infoFormat", MSG_CHATTYTUNES_DEFAULTINFOSTRING, ""],

        ["idleString", MSG_CHATTYTUNES_DEFAULTIDLESTRING, ""],

        ["maxDoubleTakes", 20, ""],

        ["maxSearchErrors", 10, ""]

    ];

    plugin.prefary = plugin.prefary.concat(myprefs);

    

    //Hack the prefs help and label :D

    client._defaultBundle = client.defaultBundle;

    client._defaultBundleCount = 1;

    client._defaultBundleCurrent = plugin.bundle;

    delete client.defaultBundle;



    client.__defineGetter__("defaultBundle", getClientStrBundle);



    //Response maps for those who stylize Chatzilla.

    client.responseCodeMap["tunes"] = "[TUNES]";

    client.responseCodeMap["tunes-error"] = "[TUNES]";

    client.responseCodeMap["tunes-play"] = "[TUNES]";

    client.responseCodeMap["tunes-pause"] = "[TUNES]";

    client.responseCodeMap["tunes-stop"] = "[TUNES]";

    

    //Replace regexp for title, player, position, length

    plugin.re = /\$\((player)\)|\$\((track)\)|\$\((length)\)|\$\((position)\)/gi;



    //Event pump. Thanks to Silver.

    plugin.newEvent =

    function _newEvent (set, type, obj, fun) 

    {

        return new CEvent(set, type, obj, fun);

    }



    //Check if FoxyTunes exists. Only needs to be run on init, 

    //seeing as nothing is going to happen while Cz is open.

    if(client.host == "Firefox" || client.host == "Mozilla")

    {

        var c = 0;

        for (var i in Components.classes)

        {

            if(i.match(/^@foxytunes.org\/FoxyTunes/))

                c++;

        }

        if(c != 0) 

        {

            var prefServId = "@mozilla.org/preferences-service;1";

            var foxyTunesPrefService = newObject(prefServId, "nsIPrefService");

            plugin.fTPrefs = foxyTunesPrefService.getBranch("foxytunes.");

            plugin.playerClass = plugin.fTPrefs.getCharPref("player_class");

            plugin.playerId = plugin.playerClass.split(/(\/|;)/)[4];

            plugin.foxyTunesInstalled = true;

        }

    }

}



plugin.enable =

function enablePlugin ()

{

    var cmdManager = client.commandManager;

    //Commands.

    plugin.cmdlist = [

        ["np", cmdCTGeneral, CMD_CONSOLE | CMD_NEED_CHAN],

        ["ctplay", cmdCTPlay, CMD_CONSOLE, "[<terms>]"],

        ["ctsearch", cmdCTGeneral, CMD_CONSOLE],

        ["ctpause", cmdCTGeneral, CMD_CONSOLE],

        ["ctstop", cmdCTGeneral, CMD_CONSOLE],

        ["ctplaying", cmdCTGeneral, CMD_CONSOLE],

        ["ctnext", cmdCTGeneral, CMD_CONSOLE],

        ["ctprev", cmdCTGeneral, CMD_CONSOLE],

        ["ctmute", cmdCTGeneral, CMD_CONSOLE],

        ["ctshowplayer", cmdCTGeneral, CMD_CONSOLE],

        ["cthideplayer", cmdCTGeneral, CMD_CONSOLE],

        ["ctplayfile", cmdCTGeneral, CMD_CONSOLE],

        ["ctinsertsong", cmdCTInsertSong, 0],

        ["ctoptions", cmdCTOptions, 0],

        ["ctmenuinsertsong", "ctinsertsong", 0]

    ];

    plugin.cmdlist.stringBundle = plugin.bundle; 

    plugin.cmdlist = cmdManager.defineCommands(plugin.cmdlist);

    cmdManager.argTypes["terms"] = cmdManager.argTypes["rest"];



    plugin.prefManager.defaultBundle = plugin.bundle;



    //That damn menu.

    var menuPresence = ("mainmenu:chatty" in client.menuManager.menuSpecs);

    var playerId = plugin.playersMap[plugin.playerId];

    var aKey = playerId.accessKey;

    var menuLabel = playerId.shortName.replace(aKey, "&" + aKey);

    client.menuManager.menuSpecs["mainmenu:chatty"] = {

        label: menuLabel,

        getContext: getTunesContext,

        items: [

            ["ctplay",           {label: MSG_CHATTYTUNES_MENUPLAY}],

            ["ctstop",           {label: MSG_CHATTYTUNES_MENUSTOP}],

            ["ctpause",          {label: MSG_CHATTYTUNES_MENUPAUSE}],

            ["ctnext",           {label: MSG_CHATTYTUNES_MENUNEXT}],

            ["ctprev",           {label: MSG_CHATTYTUNES_MENUPREV}],

            ["-"],

            ["ctsearch",         {label: MSG_CHATTYTUNES_MENUSEARCH}],

            ["-"],

            ["ctoptions",        {label: MSG_CHATTYTUNES_MENUOPTIONS}],

            ["-",                {visibleif: "cx.currentSong != null"}],

            ["ctmenuinsertsong", {format: "$currentSong", label: "ChattyTunes",

                                  style: "font-weight: bold", 

                                  visibleif: "cx.currentSong != null"}]

        ]

    };

    if (!menuPresence && client.initialized)

        client.menuManager.createMenu(document.getElementById("mainmenu"), null,

                                      "mainmenu:chatty", "mainmenu:chatty");

    if (!("outputFilters" in client))

        client.outputFilters = new Object();

    client.outputFilters["chattytunes"] = { func: textFilter, enabled: true };



    //Disable plugin if FT doesn't exist, or if CZ is running in XULRunner.

    if (plugin.foxyTunesInstalled)

    {

        display(getMsg(MSG_CHATTYTUNES_ENABLED, plugin.version));

    }

    else

    {

        if(client.host == "XULrunner")

            client.display(MSG_CHATTYTUNES_XULRUNNER, CT_ERROR);

        else

            client.display(getMsg(MSG_CHATTYTUNES_NOFOXYTUNES, client.host),

                           CT_ERROR);

        plugin.enabled = false;

        plugin.prefs["enabled"] = false;

        plugin.disable();

        return false;

    }

    if (!checkForNewFT())

    {

        client.display(getMsg(MSG_CHATTYTUNES_NOFOXYTUNES, client.host),

                       CT_ERROR);

        plugin.enabled = false;

        plugin.prefs["enabled"] = false;

        plugin.disable();

        return false;

    }

    //Commands:

    plugin.chattyTunesCmds = {

        ctmute: plugin.foxyTunes.ToggleMute,

        cthideplayer: plugin.foxyTunes.HidePlayer,

        ctshowplayer: plugin.foxyTunes.ShowPlayer,

        ctplayfile: plugin.foxyTunes.PlayFile,

        ctpause: function () {

            plugin.foxyTunes.Pause();

            display(MSG_CHATTYTUNES_PAUSED, CT_PAUSE);

        },

        ctstop: function () {

            plugin.foxyTunes.Stop();

            display(MSG_CHATTYTUNES_STOPPED, CT_STOP);

        },

        ctnext: function () {

            plugin.foxyTunes.Next();

            timeOutNPMsg();

        },

        ctprev: function () {

            plugin.foxyTunes.Previous();

            timeOutNPMsg();

        },

        ctplaying: function () {

            display(getInfoString(plugin.prefs["infoFormat"]), CT_PLAY);

        },

        ctsearch: function () {

            var searchString = prompt(MSG_CHATTYTUNES_SEARCHFORM, "");

            if (searchString)

                dispatch("ctplay " + searchString);

        },

        np: function () {

            dispatch(getInfoString(plugin.prefs["NPFormat"]));

        }

    }

    return true;

}



plugin.disable =

function disablePlugin ()

{

    plugin.chattyTunesCmds = null;

    client.outputFilters["chattytunes"].enabled = false;

    client.commandManager.removeCommands(plugin.cmdlist);

    display(getMsg(MSG_CHATTYTUNES_DISABLED, plugin.version));

    return true;

}



function getClientStrBundle () 

{

    if (client._defaultBundleCount == 0) 

        client._defaultBundleCurrent = client._defaultBundle;

    else

        client._defaultBundleCount--; 

    return client._defaultBundleCurrent; 

}



function getTunesContext ()

{

    checkForNewFT();

    cx = new Object();

    cx.currentSong = plugin.foxyTunes.GetCurrentTrackTitle();

    if(cx.currentSong === "" || plugin.foxyTunes == null)

        cx.currentSong = null;

    if ("encoding" in plugin && plugin.encoding != "default" && cx.currentSong != null)

        cx.currentSong = toUnicode(cx.currentSong, plugin.encoding);

    else

        cx.currentSong = toUnicode(cx.currentSong, "UTF-8");

    return cx;

}



//Helps recreate the FoxyTunes object if the player class is changed.

function checkForNewFT ()

{

    var playerClass = plugin.fTPrefs.getCharPref("player_class");

    var encoding = plugin.fTPrefs.getCharPref("character_encoding");

    var identical = ("encoding" in plugin && "playerClass" in plugin &&

                     plugin.playerClass == playerClass && plugin.encoding == encoding);

    if ("foxyTunes" in plugin && plugin.foxyTunes != null && identical)

        return true;

    if ("encoding" in plugin && plugin.encoding != encoding)

        display(MSG_CHATTYTUNES_CHANGEDENCODING, CT_INFO);

    if ("playerClass" in plugin && plugin.playerClass != playerClass)

        display(MSG_CHATTYTUNES_CHANGEDPLAYER, CT_INFO);

    plugin.encoding = plugin.fTPrefs.getCharPref("character_encoding");

    plugin.playerClass = plugin.fTPrefs.getCharPref("player_class");

    try 

    {

        plugin.foxyTunes = Components.classes[plugin.playerClass].getService();

        plugin.foxyTunes = plugin.foxyTunes.QueryInterface(Components.interfaces.mozIFoxyTunes);

        plugin.playerId = plugin.playerClass.split(/(\/|;)/)[4];

        updateMenuLabel();

        if ("chattyTunesCmds" in plugin && plugin.chattyTunesCmds != null)

        {

            plugin.chattyTunesCmds["ctmute"] = plugin.foxyTunes.ToggleMute;

            plugin.chattyTunesCmds["cthideplayer"] = plugin.foxyTunes.HidePlayer;

            plugin.chattyTunesCmds["ctshowplayer"] = plugin.foxyTunes.ShowPlayer;

            plugin.chattyTunesCmds["ctplayfile"] = plugin.foxyTunes.PlayFile;

        }

        return true;

    } 

    catch (ex) 

    {

        display(getMsg(MSG_CHATTYTUNES_ERRORLOADING, ex), CT_ERROR);

        return false;

    }

}



function getInfoString (message)

{

    message = message.replace(plugin.re, replaceVars);

    if (client.prefs["outgoing.colorCodes"] = true)

        message = replaceColorCodes(message);

    return message;

}



function textFilter (msg, msgtype)

{

    if (!plugin.enabled)

        return msg;

    try

    {

        if (!checkForNewFT())

            return msg;

    }

    catch (ex)

    {

        return msg;

    }

    msg = msg.replace(plugin.re, replaceVars);

    return msg;

}



function getTimeString (seconds) 

{

    if (!parseInt(seconds, 10))

        return "0:00";

    seconds = parseInt(seconds);

    if(seconds == 4294967 || seconds == 4294967295) //Winamp & QCD

        seconds = 0;

    var time_hours = Math.floor(seconds / 3600);

    var time_seconds = seconds % 60;

    var time_minutes = ((seconds % 3600) - time_seconds) / 60;

    if (time_seconds < 10)

        time_seconds = "0" + time_seconds;

    var timeStr = time_minutes + ":" + time_seconds;

    if (time_hours > 0)

    {

        if (time_minutes < 10)

            timeStr = "0" + timeStr;

        timeStr = time_hours + ":" + timeStr;

    }

    return timeStr;

}



function replaceVars (match, player, track, length, position)

{

    if (player)

        return plugin.playersMap[plugin.playerId].longName;

    else if (track)

        return getTrackTitle();

    else if (length)

        return getTrackLength();

    else if (position)

        return getTrackPos();

}



function getTrackTitle ()

{

    try 

    {

        var song = plugin.foxyTunes.GetCurrentTrackTitle();

    } 

    catch (ex) 

    {

        var song = null;

    }

    if (song === "" || song == null)

        song = plugin.prefs["idleString"];

    else if ("encoding" in plugin && plugin.encoding != "default")

        song = toUnicode(song, plugin.encoding);

    else

        song = toUnicode(song, "UTF-8");

    return song;

}



function getTrackLength ()

{

    try 

    {

        return getTimeString(plugin.foxyTunes.GetCurrentTrackLength());

    } 

    catch (ex) 

    {

        return "0:00";

    }

}



function getTrackPos ()

{

    try 

    {

        return getTimeString(plugin.foxyTunes.GetCurrentTrackPosition());

    } 

    catch (ex) 

    {

        return "0:00";

    }

}



function updateMenuLabel () 

{

    if (!client.initialized)

    {

        return true;

    }

    //this won't work when the menu is not there (at startup), ergo above.

    var menu = document.getElementById("mainmenu:chatty");

    var playerId = plugin.playersMap[plugin.playerId];

    menu.setAttribute("label", playerId.shortName);

    menu.setAttribute("accesskey", playerId.accessKey);

}

    



//Commands are going here because putting them in another file was a hassle.

//If anybody can work it out (and have it work), do tell us.



function timeOutNPMsg ()

{

    setTimeout("display('" + getInfoString(plugin.prefs["infoFormat"]) + 

               "', 'tunes-play')", 1500);

}



function cmdCTGeneral (e)

{

    checkForNewFT();

    var commandname = e.command.name.toLowerCase();

    if (commandname in plugin.chattyTunesCmds)

    {

        try

        {

            plugin.chattyTunesCmds[commandname]();

        }

        catch(ex)

        {

            var pms = [commandname, ex.message];

            display(getMsg(MSG_CHATTYTUNES_ERRORGENERAL, pms), MT_ERROR);

        }

        return;

    }

    display(getMsg(MSG_NO_CMDMATCH, commandname), MT_ERROR);

}



function cmdCTInsertSong (e) {

    checkForNewFT();

    var song = getTrackTitle();

    var input = document.getElementById("input");

    var text = input.value;

    if (!text) 

    {

        input.value = song;

        return;

    }



    var selStart = input.selectionStart;

    var selEnd = input.selectionEnd;

    var textBeginning = text.substring(0, selStart);

    var textEnd = text.substring(selEnd);

    if (selStart != selEnd)

    {

        textBeginning += song;

        input.value = textBeginning + textEnd;

        input.selectionEnd = input.selectionStart + (selEnd - selStart);

    } 

    else 

    {

        if (textBeginning.match(/\s$/))

        {

            textBeginning += song;

        } 

        else if (textBeginning.match(/\"$/)) 

        {

            var quoteNumber = textBeginning.split("\"");

            quoteNumber = quoteNumber.length - 1;

            if (quoteNumber % 2 == 0)

                textBeginning += " " + song;

            else

                textBeginning += song + "\"";

        } 

        else 

        {

            textBeginning += " " + song;

        }



        if (!textEnd.match(/^\s/))

            textEnd = " " + textEnd;

        input.value = textBeginning + textEnd;

    }

}



function cmdCTPlay (e)

{

    checkForNewFT();

    var searcherrors = 0;

    if (e.terms == null)

    {

        plugin.foxyTunes.Play();

        timeOutNPMsg();

    }

    else

    {

        plugin.foxyTunes.Stop();

        var searchExpression = new RegExp(e.terms, "gi");

        display(MSG_CHATTYTUNES_SEARCHING, CT_INFO);



        try 

        {

            var oldtitle = plugin.foxyTunes.GetCurrentTrackTitle();

        } 

        catch (ex) 

        {

            display(getMsg(MSG_CHATTYTUNES_ERRORORIGINALTITLE, ex), CT_ERROR);

            var oldtitle = null;

            return;

        }

        var searchfailed = false;

        var currenttitle = "";

        var lasttitle = oldtitle;

        var searcherrors = 0;

        var doubletakes = 0;



        do

        {

            plugin.foxyTunes.Next();

            try 

            {

                currenttitle = plugin.foxyTunes.GetCurrentTrackTitle();

            } 

            catch (ex) 

            {

                searcherrors++;

                currenttitle = null;

            }

            while (currenttitle == null || currenttitle == lasttitle)

            {

                doubletakes++;

                try

                {

                    currenttitle = plugin.foxyTunes.GetCurrentTrackTitle();

                } 

                catch (ex) 

                {

                    currenttitle = null;

                    searcherrors++;

                    if (searcherrors >= plugin.prefs["maxSearchErrors"])

                    {

                        display(MSG_CHATTYTUNES_ERRORMAXERRORS, CT_ERROR);

                        return;

                    }

                }

                if (doubletakes >= plugin.prefs["maxDoubleTakes"])

                {

                    display(MSG_CHATTYTUNES_ERRORDOUBLETAKES, CT_ERROR);

                    return;

                }

            }

            doubletakes = 0;

            if (currenttitle == oldtitle)

            {

                searchfailed = true;

                setTimeout(plugin.foxyTunes.Play, 2000);

            }

            else

            {

                if (currenttitle.match(searchExpression) != null)

                {

                     display(getMsg(MSG_CHATTYTUNES_FOUND, currenttitle), CT_PLAY);

                     setTimeout(plugin.foxyTunes.Play, 2000);

                     return;

                }

                lasttitle = currenttitle;

                if (searcherrors > plugin.prefs["maxSearchErrors"])

                {

                    display(MSG_CHATTYTUNES_ERRORMAXERRORS, CT_ERROR);

                    searcherrors = 0;

                    return;

                }

            }

        }

        while (!searchfailed)

        display(getMsg(MSG_CHATTYTUNES_NOTFOUND, e.terms), CT_INFO);

    }

}



function cmdCTOptions () 

{

    var options = new Object();

    options.NPFormat = plugin.prefs["NPFormat"];

    options.idleString = plugin.prefs["idleString"];

    options.infoFormat = plugin.prefs["infoFormat"];

    options.maxDoubleTakes = plugin.prefs["maxDoubleTakes"];

    options.maxSearchErrors = plugin.prefs["maxSearchErrors"];

    var windowArgs = "dialog,width=500,height=315";

    var dialogURL = plugin.urlString + "dialog/dialog.xul";

    var nickname = client.prefs["nickname"]; //Pass nickname

    var errNoPreview = MSG_CHATTYTUNES_ERRORNOPREVIEW;

    if (client.prefs["outgoing.colorCodes"] = true)

        var replFunc = replaceColorCodes;

    else

        var replFunc = function (msg) { return msg; };

    plugin.options = window.openDialog(dialogURL, "_blank", windowArgs, client,

                                       plugin, replFunc, options, nickname, 

                                       errNoPreview);

}



function setPrefs (e) 

{

    plugin.prefs["NPFormat"] = e.NPFormat;

    plugin.prefs["infoFormat"] = e.infoFormat;

    plugin.prefs["idleString"] = e.idleString;

    plugin.prefs["maxDoubleTakes"] = e.maxDoubleTakes;

    plugin.prefs["maxSearchErrors"] = e.maxSearchErrors;

}



function replaceLiveEditVars (e) 

{

    e.message = getInfoString(e.message);

    plugin.options.fillInVars(e);

}